﻿using System;
using System.Collections;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Security.AccessControl;
using System.Security.Policy;
using System.Text;
using System.Threading.Tasks;
using ValidationExample.Attributes;


namespace ValidationExample.ViewModel
{
    public class MainWindowViewModel : INotifyPropertyChanged, INotifyDataErrorInfo
    {

        // fields

        private readonly ConcurrentDictionary<string, List<ValidationResult>> _errors =
            new ConcurrentDictionary<string, List<ValidationResult>>();

        private string _url;

        // property

        [ReachableUrl]
        [UrlAttribute]
        public string Url
        {
            get { return _url; }
            set
            {
                if (value != _url)
                {
                    _url = value;
                    OnPropertyChanged();
                    ValidateUrl(_url);
                }
            }
        }


        // Validation methods

        private async void ValidateUrl(string urlString, [CallerMemberName] String propertyName = "")
        {
            await Task.Factory.StartNew(() =>
            {
                // Remove the existing errors for this property
                List<ValidationResult> existingErrors;
                _errors.TryRemove(propertyName, out existingErrors);

                // Check for valid URL
                var results = new List<ValidationResult>();
                var vc = new ValidationContext(this) { MemberName = propertyName };
                if (Validator.TryValidateProperty(urlString, vc, results) || results.Count <= 0) return;
                _errors.AddOrUpdate(propertyName, new List<ValidationResult>
                { new  ValidationResult(results[0].ErrorMessage)},
                    (key, existingVal) => new List<ValidationResult> 
                    { new ValidationResult(results[0].ErrorMessage)});
                NotifyErrorsChanged(propertyName);
            });
        }



        // Data Validation Interface

        public IEnumerable GetErrors(string propertyName)
        {
            List<ValidationResult> propertyErrors = null;
            _errors.TryGetValue(propertyName, out propertyErrors);
            return propertyErrors;
        }

        public bool HasErrors { get { return _errors.Count > 0; } }

        public event EventHandler<DataErrorsChangedEventArgs> ErrorsChanged;

        protected virtual void NotifyErrorsChanged([CallerMemberName] String propertyName = "")
        {
            var e = ErrorsChanged;
            if (e == null) return;
            e(this, new DataErrorsChangedEventArgs(propertyName));
        }

        // Property Changed

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            var handler = PropertyChanged;
            if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
        }

    }
}
